@@ -1,12 +1,12 @@
-# 
-# auth_ldap.pl - authenticate user with LDAP
-# 
-# 2002/01/27 Ivan Cerrato - pengus@libero.it
 #
-# Based on auth_unix.pl written by tung@turtle.ee.ncku.edu.tw 
+# auth_ldap.pl - authenticate user with LDAP
+#
+# 2002/04/10 Kelson Vibber - kelson@speed.net (fixed check_userpassword)
+# 2002/01/27 Ivan Cerrato - pengus@libero.it
 #
 
 my $ldapHost = "HOSTNAME";	# INSERT THE LDAP SERVER IP HERE.
+my $ou = "ou=People";		# INSERT THE LDAP ORGANIZATIONAL UNIT HERE.
 my $cn = "cn=LOGIN";		# INSERT THE LDAP USER HERE.
 my $dc1 = "dc=DC1";		# INSERT THE FIRST DC HERE.
 my $dc2 = "dc=DC2";		# INSERT THE SECOND DC HERE.
@@ -14,6 +14,7 @@ my $pwd = "PASSWORD";		# INSERT THE LDAP PASSWORD HERE.
 
 ################### No configuration required from here ###################
 
+use strict;
 use Net::LDAP;
 
 my $ldapBase = "$dc1, $dc2";
@@ -25,7 +26,7 @@ $ldap->bind ( dn        =>      $dn,
 
 sub get_userinfo {
    my $user=$_[0];
-   my ($uid, $gid, $realname, $homedir);
+   my ($uid, $gid, $gecos, $homedir);
 
    my $list = $ldap->search (
                              base    => $ldapBase,
@@ -37,17 +38,18 @@ sub get_userinfo {
 	return -1;
 	}
    else {
-	$entry = $list->entry(0);
+	my $entry = $list->entry(0);
 
+        $gecos = $entry->get_value("gecos");
         $uid = $entry->get_value("uidNumber");
         $gid = $entry->get_value("gidNumber");
-        $gecos = $entry->get_value("gecos");
-        $home = $entry->get_value("homeDirectory");
+        $homedir = $entry->get_value("homeDirectory");
 
-	return($gecos, $uid, $gid, $home);
+	return($gecos, $uid, $gid, $homedir);
 	}
 }
 
+
 sub get_userlist {      # only used by checkmail.pl -a
    my @userlist=();
 
@@ -59,7 +61,7 @@ sub get_userlist {      # only used by checkmail.pl -a
 
    my $num = $list->count;
 
-   for ($i = 0; $i < $num; $i++) {
+   for (my $i = 0; $i < $num; $i++) {
 	my $entry = $list->entry($i);
 	push (@userlist, $entry->get_value("uid"));
 	}
@@ -67,47 +69,36 @@ sub get_userlist {      # only used by checkmail.pl -a
    return (@userlist);
 }
 
+
 #  0 : ok
 # -2 : parameter format error
 # -3 : authentication system/internal error
 # -4 : password incorrect
-
 sub check_userpassword {
    my ($user, $password)=@_;
 
    return -2 unless ( $user ne "" && $password ne "");
 
-   my $list = $ldap->search (
-                             base    => $ldapBase,
-                             filter  => "(&(objectClass=posixAccount)(uid=$user))",
-                             attrs   => ['userPassword']
-                             );
-
-   if ($list->count eq 0) {
-        return -4;
-        }
-   else {
-	my $entry = $list->entry(0);
-	$tmp_pwd = $entry->get_value("userPassword");
-
-	$c_pwd = substr($tmp_pwd, 7, 13);
-	$salt = substr($c_pwd, 0, 2);
-
-	if ($c_pwd eq crypt($password, $salt)) {
-        	return 0;
-		}
+	my $dn = "uid=$user, $ou, $dc1, $dc2";
+	# Attempt to bind using the username and password provided.
+	# (For a secure LDAP config, only auth should be allowed for
+	# any user other than self and rootdn.)
+	my $mesg = $ldap->bind ( dn        =>      $dn,
+        	password  =>      $password);
+	if( $mesg->code == 0 ) {
+		return 0;
+	}
 	else {
 		return -4;
-		}
-        }   
+	}
 }
 
+
 #  0 : ok
 # -1 : function not supported
 # -2 : parameter format error
 # -3 : authentication system/internal error
 # -4 : password incorrect
-
 sub change_userpassword {
    my ($user, $oldpassword, $newpassword)=@_;
    my ($u, $p, $misc, $encrypted);
@@ -136,4 +127,3 @@ sub change_userpassword {
 }
 
 1;
-
