@@ -1,26 +1,27 @@
-# 
+#
 # auth_unix.pl - authenticate user with unix password
 #
-# $unix_passwdfile : the location of the file containing all usernames 
-#                    and their corresponding encrypted passwords. 
+# $unix_passwdfile : the location of the file containing all usernames
+#                    and their corresponding encrypted passwords.
 # ------------------------------   --------------------------------
 # platform                         passwdfile
 # ------------------------------   --------------------------------
 # Shadow Passwd (linux/solaris)    /etc/shadow
 # FreeBSD                          /etc/master.passwd
+# Mac OS X                         /usr/bin/nidump passwd .|
 # NIS/YP                           /usr/bin/ypcat passwd|
 # NIS+                             /usr/bin/niscat passwd.org_dir|
-# else....                         /etc/passwd
+# else...                          /etc/passwd
 # ------------------------------   --------------------------------
 #
-# $unix_passwdmkdb : The command executed after any password modification 
+# $unix_passwdmkdb : The command executed after any password modification
 #                    to update the changes of passwdfile to passwd database.
 # ------------------------------   --------------------------------
 # platform                         passwd mkdb command
 # ------------------------------   --------------------------------
 # Free/Net/OpenBSD                 /usr/sbin/pwd_mkdb
-# Linux	                           none
-# Solaris                          none
+# Linux/Solaris	                   none
+# else...                          none
 # ------------------------------   --------------------------------
 #
 # 2001/12/20 tung@turtle.ee.ncku.edu.tw
@@ -31,6 +32,9 @@ my $unix_passwdmkdb="/usr/sbin/pwd_mkdb";
 
 ################### No configuration required from here ###################
 
+use strict;
+use Fcntl qw(:DEFAULT :flock);
+
 sub get_userinfo {
    my $user=$_[0];
    my ($uid, $gid, $realname, $homedir) = (getpwnam($user))[2,3,6,7];
@@ -47,7 +51,7 @@ sub get_userinfo {
 
 sub get_userlist {	# only used by checkmail.pl -a
    my @userlist=();
-   my $line;   
+   my $line;
 
    # a file should be locked only if it is local accessable
    filelock("$unix_passwdfile", LOCK_SH) if ( -f $unix_passwdfile);
@@ -86,7 +90,7 @@ sub check_userpassword {
 
    if ($u eq $user && crypt($password,$p) eq $p) {
       return 0;
-   } else { 
+   } else {
       return -4;
    }
 }
@@ -154,8 +158,9 @@ sub change_userpassword {
       }
    } else {
       # automic update passwd by rename
-      chown(0,0, "$unix_passwdfile.tmp.$$");
-      chmod(0600, "$unix_passwdfile.tmp.$$");
+      my ($fmode, $fuid, $fgid) = (stat($unix_passwdfile))[2,4,5];
+      chown($fuid, $fgid, "$unix_passwdfile.tmp.$$");
+      chmod($fmode, "$unix_passwdfile.tmp.$$");
       rename("$unix_passwdfile.tmp.$$", $unix_passwdfile) || goto authsys_error;
    }
    filelock("$unix_passwdfile", LOCK_UN);
