@@ -1,31 +1,34 @@
 #
-# encode_base64, decode_base64 and decode_qp are blatantly snatched 
-# from parts of the MIME-Base64 Perl modules. 
+# encode_base64, decode_base64 and decode_qp are blatantly snatched
+# from parts of the MIME-Base64 Perl modules.
 #
-# the encoding/decoding speed would be much faster if you install 
+# the encoding/decoding speed would be much faster if you install
 # MIME-Base64 perl module (MIME-Base64-2.12.tar.gz) with XS support
 #
+use strict;
 
 if (eval "require MIME::Base64") {
    import MIME::Base64;
-   push @Uses, "B$MIME::Base64::VERSION";
+   push @::Uses, "B$MIME::Base64::VERSION";
 } else {
    eval q{
 
-sub encode_base64 
+sub encode_base64
 {
+   my $str = shift;
    my $res = "";
    my $eol = "\n";
 
-   pos($_[0]) = 0;      # thanks, Andreas!
-   while ($_[0] =~ /(.{1,45})/gs) {
+   pos($str) = 0;      # thanks, Andreas!
+
+   while ($str =~ /(.{1,45})/gs) {
       $res .= substr(pack('u', $1), 1);
       chop($res);
    }
    $res =~ tr|` -_|AA-Za-z0-9+/|;
 
    # Fix padding at the end:
-   my $padding = (3 - length($_[0]) % 3) % 3;
+   my $padding = (3 - length($str) % 3) % 3;
    $res =~ s/.{$padding}$/'=' x $padding/e if $padding;
 
    # Break encoded string into lines of no more than 76 characters each:
@@ -56,7 +59,7 @@ sub decode_base64
 
 if (eval "require MIME::QuotedPrint") {
    import MIME::QuotedPrint qw(decode_qp);
-   push @Uses, "Q$MIME::QuotedPrint::VERSION";
+   push @::Uses, "Q$MIME::QuotedPrint::VERSION";
 } else {
    eval q{
 
@@ -73,7 +76,6 @@ sub decode_qp
 } #if
 
 
-
 sub decode_mimewords {
     my $encstr = shift;
     my %params = @_;
